# frozen_string_literal: true

class ComplexModality < ActiveRecord::Base
  include ComplexHelper
  include HyraxHelper
  include S3FileHandleable

  attr_accessor :is_imported

  belongs_to :complex_session, foreign_key: 'parent_source_identifier', primary_key: 'source_identifier', optional: true

  validates :source_identifier, presence: true, uniqueness: true

  attribute :source_identifier, :string, default: -> { SecureRandom.uuid }
  validates :s3_folder_name, presence: { message: ->(object, data) { I18n.t('rdms.errors.complex_modality.title.blank', value: object.s3_folder_name) } }
  validates_length_of :s3_folder_name, within: 3..63, too_short: ->(object, data) { I18n.t('rdms.errors.complex_modality.title.length', value: object.s3_folder_name) }, too_long: ->(object, data) { I18n.t('rdms.errors.complex_modality.title.length', value: object.s3_folder_name) }
  validates :modality, presence: { message: "Modality can't be blank" }
  validates :s3_folder_name, uniqueness: { scope: :parent_source_identifier, message: ->(object, data) { I18n.t('rdms.errors.complex_modality.title.uniqueness', value: object.s3_folder_name) } }, unless: -> { self.is_imported }

  before_validation :set_s3_folder_name
  after_save :save_metadata_as_json_in_s3, :save_meta_json_in_s3, :update_crc1280_experiment_date_modified_and_files_data, :reindex_parent_work
  after_commit :fix_duplication_of_title, on: :create
  before_destroy :clear_s3_and_all_associated_objects

  delegate :crc1280_experiment, to: :complex_session
  delegate :complex_subject, to: :complex_session

  scope :sort_by_title, ->(source_identifier) {
    where(parent_source_identifier: source_identifier)
    .sort_by { |item| item.modality_title.downcase.gsub(/[^a-zA-Z0-9]/, '0').split(/(\d+)/).map { |s| s.match(/\d+/) ? s.to_i : s } }
  }

  def file_sets
    @file_sets ||= FileSet.where(for_complex_type: ["ComplexModality"], for_complex_identifier: [self.source_identifier])
  end

  def s3_object_prefix
    "#{crc1280_experiment.id}/#{complex_subject.s3_folder_name}/#{complex_session.s3_folder_name}/#{s3_folder_name}/"
  end

  private

  def set_s3_folder_name
    self.s3_folder_name = sanitize_title(modality_title)
  end

  def fix_duplication_of_title
    return true  unless self.is_imported

    suffix = 1
    new_title = self.modality_title

    while ComplexModality.where(modality_title: new_title, parent_source_identifier: complex_session.source_identifier).where.not(id: self.id).present?
      new_title = "#{self.modality_title}_#{suffix}"
      suffix += 1
    end

    self.modality_title = new_title
    self.save
  end

  def save_metadata_as_json_in_s3
    s3 = S3StorageService.new
    s3.init_client
    bucket_name = s3.sanitise_name(ENV.fetch("S3_FILE_UPLOAD_BUCKET"))

    s3.add_content(bucket_name, "#{crc1280_experiment.id.to_s}/#{complex_subject.s3_folder_name}/#{complex_session.s3_folder_name}/#{s3_folder_name}/system_metadata.json", to_json)
    s3.add_content(bucket_name, "#{crc1280_experiment.id.to_s}/#{complex_subject.s3_folder_name}/#{complex_session.s3_folder_name}/#{s3_folder_name}/metadata.json", filter_metadata_json_file(self))
  end

  def reindex_parent_work
    Hyrax.index_adapter.save(resource: crc1280_experiment)
  end
end
